# -*- coding: utf-8 -*-
from setuphelpers import *
import psutil
import time
import threading

"""
  {
   "key":"{ce044972-8c61-4a19-8f29-08e5c75f2674}",
   "name":"SeaDrive 1.0.12",
   "version":"1.0.12",
   "install_date":"",
   "install_location":"",
   "uninstall_string":"\"C:\\ProgramData\\Package Cache\\{ce044972-8c61-4a19-8f29-08e5c75f2674}\\seadrive.exe\"  /uninstall",
   "publisher":"Seafile ltd.",
   "system_component":0,
   "win64":false
  }
  }
   "key":"{7FD14F30-762C-48CF-830B-063CD4F99188}",
   "name":"SeaDrive 3.0.8",
   "version":"3.0.8",
   "install_date":"2024-03-18 00:00:00",
   "install_location":"",
   "uninstall_string":"MsiExec.exe /X{7FD14F30-762C-48CF-830B-063CD4F99188}",
   "publisher":"HaiWenHuZhi ltd.",
   "system_component":0,
   "win64":true
  }
"""
# seadriveRoot
# old=C:\Users\username\seadrive\data
# new=C:/Users/username/seadrive_root


def install():
    bin_name = glob.glob("seadrive-*.msi")[0]
    uninstall_seadrive_v1()
    install_msi_if_needed(
        bin_name,
        min_version=control.get_software_version(),
        timeout=600,
    )

    # Clear the uninstallkeys to avoid the no-code audit and uninstall
    uninstallkey.clear()


def session_setup():
    print("Disabling: auto-update check and configuring base options")
    registry_setstring(HKEY_CURRENT_USER, r"SOFTWARE\SeaDrive", "ShellExtDisabled", "1")
    registry_setstring(HKEY_CURRENT_USER, r"SOFTWARE\SeaDrive\Seafile Drive Client\Language", "current", "en")
    registry_setstring(HKEY_CURRENT_USER, r"SOFTWARE\SeaDrive\Seafile Drive Client\Misc", "SparkleAlreadyEnableUpdateByDefault", "true")
    registry_setstring(HKEY_CURRENT_USER, r"SOFTWARE\SeaDrive\Seafile Drive Client\WinSparkle", "CheckForUpdates", "0")
    registry_setstring(HKEY_CURRENT_USER, r"Software\SeaDrive\Seafile Drive Client\Behavior", "hideMainWindowWhenStarted", "true")
    #registry_setstring(HKEY_CURRENT_USER, r"SOFTWARE\SeaDrive\Seafile Drive Client\Settings", "diskLetter", "S:")
    
    #Since version 3, there is no more drive letter assigned. Here is how to configure it.
    mount_point = registry_readstring(HKEY_CURRENT_USER, r"SOFTWARE\SeaDrive\Seafile Drive Client\Settings", "seadriveRoot")
    if isdir(mount_point):
        for folder in glob.glob(f'{mount_point}\*\Shared with groups'):
            if isdir(folder):
                registry_setstring(HKEY_CURRENT_USER, r"SOFTWARE\Microsoft\Windows\CurrentVersion\Run", "Seadrive_letter" , rf'''C:\Windows\System32\cmd.exe /C "subst S: {folder.split('Shared with groups')[0]}"''')
                #run(f'subst S: {folder.split("Shared with groups")[0]}')
                break
    
    #Seadrive is now installed in programfiles, not programfiles32
    registry_setstring(HKEY_CURRENT_USER, r"SOFTWARE\Microsoft\Windows\CurrentVersion\Run", "Seadrive", r"C:\Program Files\SeaDrive\bin\seadrive-gui.exe")


def audit():
    # Declaring local variables
    audit_status = "OK"
    app_check = installed_softwares("SeaDrive")
    if app_check:
        installed_version = app_check[0]["version"]
        control.name = app_check[0]["name"]
    else:
        installed_version = ""
        control.name = control.package.split("-", 1)[-1].replace("-", " ").title()
    audit_version = False

    # Auditing software
    if not installed_version:
        print(f"{control.name} is not installed.")
        audit_status = "ERROR"
    elif audit_version and Version(installed_version, 4) < Version(control.get_software_version(), 4):
        print(f"{control.name} ({installed_version}) installed version should be: {control.get_software_version()}")
        audit_status = "WARNING"
    else:
        print(f"{control.name} is installed." if installed_version in control.name else f"{control.name} ({installed_version})  is installed.")
        audit_status = "OK"

    return audit_status


def uninstall():
    # "C:\Program Files\SeaDrive\bin\seadrive-gui.exe" --remove-user-data
    for app_check in installed_softwares("SeaDrive"):
        try:
            run(uninstall_cmd(app_check["key"]), timeout=60)
        except:
            killalltasks("seadrive-gui.exe")
            killalltasks(ensure_list(control.impacted_process))
            wait_uninstallkey_absent(keywords="SeaDrive")
            time.sleep(60)
            if installed_softwares("SeaDrive"):
                error("Uninstalling SeaDrive was unsuccessful.")


def uninstall_seadrive_v1():
    app_uninstallkey = "{82442898-8FE9-44E8-8F50-D2EACBB80848}"
    # Uninstalling the SeaDrive dependency called Dokan Library
    for uninstall in installed_softwares("Dokan Library"):
        print("Removing: %s" % uninstall["name"])
        run(uninstall_cmd(uninstall["key"]))

    # Uninstalling SeaDrive
    for uninstall in installed_softwares(name="SeaDrive 1"):
        # if uninstall["key"] == app_uninstallkey:
        #     continue
        print("Removing: %s" % uninstall["name"])
        t = threading.Thread(target=killseadrive)
        t.start()
        run(uninstall_cmd(uninstall["key"]))


def killseadrive():
    time.sleep(15)
    killalltasks("seadrive-gui.exe")
    time.sleep(15)
    killalltasks("seadrive-gui.exe")
    time.sleep(15)
    killalltasks("seadrive-gui.exe")
